/*
Title:			servo2313.c
Author:			Windell Oskay
Date Created:   10/13/07
Last Modified:  10/13/07
Purpose: Simple control of 6 RC servo motors

We will want to run with the 8 MHz internal RC oscillator,
and with the clock prescaler turned off (divide by 1)

Count: System clock.
Servo PWM outputs: pins B0 - B5

"Roller" program on pin B0
LED output on D5

 This code is distributed under the GNU Public License
		which can be found at http://www.gnu.org/licenses/gpl.txt

*/

#include <avr/io.h>  
#include <avr/interrupt.h>
//#include "global.h"

// Global variables
volatile unsigned short BITPHASE; 
volatile unsigned short PulseDone; 

/*
#ifndef SetBit
	#define SetBit(register,bitnumber)	   register |= (BV(bitnumber))
#endif
*/

SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	PORTB = (1 << BITPHASE);
}

SIGNAL(SIG_OUTPUT_COMPARE1B)
{
	PORTB = 0;
	PulseDone = 1;
}
	
//	asm("cli");		// DISABLE global interrupts
//	asm("sei");		// ENABLE global interrupts
	
	
/*
// Timer/Counter1 Compare Match A 
#define TIMER1_COMPA_vect		_VECTOR(11)
#define SIG_OUTPUT_COMPARE1A		_VECTOR(11)

// Timer/Counter1 Compare Match B 
#define TIMER1_COMPB_vect		_VECTOR(12)
#define SIG_OUTPUT_COMPARE1B		_VECTOR(12)
*/


int main (void)
{ 

asm("cli");		// DISABLE global interrupts

unsigned int TimeOffset;	// Initial Delay (1000 cycles?)
unsigned int positions[6];  // Values should range 0 - 1000.
unsigned int targets[6];  //(where are we going?) Values should range 0 - 1000.
unsigned int direction[6];
unsigned int moveCount;
unsigned short i, j, PosTemp, phase;
 
//uint32_t longwaittimer;
unsigned int longwaittimer;

PORTB = 0;		//All of port B should be off.
DDRB =  63U;	// #0011 1111; PB0 - PB6 are our outputs

PORTD = 0;
DDRD =	32U;	// Pin D5 is our LED output

//PORTD = 32;

//Rate of MCU clock, in MHz-- use a crystal for best results.
#define clockrate 8



TimeOffset = 1000;

positions[0] = 100;
positions[1] = 200;
positions[2] = 400;
positions[3] = 600;
positions[4] = 800;
positions[5] = 900;

targets[0] = 0;
targets[1] = 0;
targets[2] = 0;
targets[3] = 0;
targets[4] = 0;
targets[5] = 0;

direction[0] = 0;
direction[1] = 0;
direction[2] = 0;
direction[3] = 0;
direction[4] = 0;
direction[5] = 0;




 phase = 0;
 
 //Phases:  Forward 0, 1, 2, Reverse 3, 4, 5

positions[0] = 220;	// Initial Value-- should be about stopped.
						// You will have to cal the center for your particular servo!
moveCount  = 0;




////////////////////////////////////////////////////

//Timing controlled by 16-bit timer 1, a free running timer at the CPU clock rate
//
//
////////////////////////////////////////////////////


TCCR1A = 0;

//TCCR1A:
// Bits 7-4:  0000, Normal port operation, OC1A/B disconnected
// Bits 1,0: 00, WGM, normal mode


//TCCR1B =  1;		// Run clock
TCCR1B =  0;		// No clock

//TCCR1B:
// Bit 7: 0: Input noise cancel
// Bit 6: 0:  Input Capture Edge Select; positive edge trigger
// Bit 4,3: 0 (WGM)
// Bits 2,1,0:  001 Clock Select, use main clock w/o division
// Bits 2,1,0:  000 Clock Select: No clock; clock stopped.

TCCR1C =  0;		// CYA Only


////////////////////////////////////////////////////
// 8-bit Timer 0: For real-time waiting


longwaittimer = 0;
TCNT0 = 0;

TCCR0A = 0;	// "Normal operation"
//TCCR0B = 0;// Stop clock
TCCR0B = 5;// Start timer, at Clock Rate/1024.

TIFR |= 2U;  /// Clear bit 1, Ctr 0 overflow flag by writing 1.



////////////////////////////////////////////////////


																																
BITPHASE = 0;
PulseDone = 0;
	
#define DelayTime 900U			// # of 1/30 s intervals to wait before snapping

// Sadist 30-second delay!

asm("sei");		// ENABLE global interrupts
//asm("cli");		// DISABLE global interrupts
											
for (;;)  // main loop										
{

if (TIFR & 2U)		// Overflow:  at 256*1024 clock ticks = 262000, 30 Hz.
{
	longwaittimer++;		// incremented at 30 Hz.
	TIFR |= 2U;	// Clear overflow flag
}

/*
Plan:
1. Disable interrupts
2. Stop Counter
3. Clear PulseDone flag
4. Set Low & Hi compare values
5. Clear compare flags  (Necessary?)
6. Reset counter to zero.

(Clear interrupts?)

7. Enable interrupts
8. Start counter
9. Wait for PulseDone flag.
10. increment phase

Repeat
*/

TIMSK = 0; // Disable timer interrupts

TCCR1B =  0;		// Stop Counter

PulseDone = 0;

	PORTD = 0;

//Set Low & Hi compare values:

OCR1A = TimeOffset;
OCR1B = (1000 + positions[BITPHASE])*clockrate + TimeOffset;

// Reset counter to zero.
TCNT1 = 0;

TIMSK = 96U; // enable timer OC interrupts (bit 0: overflow)

TCCR1B =  1;		// Start Counter


if ((longwaittimer < 5U) || (longwaittimer > DelayTime))
	while (PulseDone == 0) {;;}			

// Could rephrase this to avoid waiting, but instead each time around
// check to see if the pulse is done, or to start this whole routine.
// Sleep until interrupt?
// Would be more precise in timing....

i = BITPHASE;

// Roller
	
	
	
	//200 slow rev
	//210 almost stpped rev
	//220: Stopped
	// 229: Almost stopped forward
	//250: Slow, forward
	
	
	
if (BITPHASE == 0) 
{	
if (longwaittimer > DelayTime)		// Wait 30 s.
{

	PORTD = 32;
	PosTemp = positions[i];
	moveCount++;
	
	
	j =  (phase > 2) + (moveCount >= 150);
	
	  
	if ( j == 1  )
		{
		PosTemp--;
		positions[0] = PosTemp;
		}
	else
	  {
		PosTemp++;
		positions[0] = PosTemp;
		}
		
	
	if (moveCount >= 300)
	{
			PORTD = 0;		//Turn LED off.
		moveCount = 0;
		longwaittimer = 0;
		positions[i] = 220;
		phase++;
		if (phase > 5)			// Three steps foward, three steps back.
			phase = 0;
	
	}
	
	
	}
}	
	
	
	
// Simple, slow zigzag mode: 
/*
if (direction[i])			// increasing
	if (PosTemp <= (unsigned int) 999U)
		 positions[i] = PosTemp + 1;
	else
		direction[i] = 0;
else						//decreasing
	if (PosTemp  >= 1U)
		 positions[i] = PosTemp - 1;
	else
		direction[i] = 1;	
*/

// Snapper:
/*
if (BITPHASE == 0)
{
PosTemp = positions[i];

if (longwaittimer > DelayTime)		// Wait 30 s.
{
	if (PosTemp < (unsigned int) 490U){
		 positions[0] = PosTemp + 1;
		 positions[1] = positions[1] + 2;
		 }
	else{
		positions[0]  = 10;
		positions[1]  = 10;
		longwaittimer = 0;
		} 
}
else
{
		positions[0]  = 10;
		positions[1]  = 10;
		}
}
*/

	
	/*
//Move towards targets:  (but throttle max rate)

if (positions[i] <= (targets[i] - 10))
	positions[i] = positions[i] + 10;

if (positions[i] >= (targets[i] + 10) )
	positions[i] = positions[i] - 10;	
*/

if ((++BITPHASE) > 5)
	BITPHASE = 0;

} // End main loop




	return 0;
}
